#include "shaderOne.h"
#include "v3.h"
#include "scene.h"

#include <iostream>

using namespace std;



CGInterface::CGInterface() : needInit(true) {};

void CGInterface::PerSessionInit() {

  CGprofile latestVertexProfile = cgGLGetLatestProfile(CG_GL_VERTEX);
  CGprofile latestGeometryProfile = cgGLGetLatestProfile(CG_GL_GEOMETRY);
  CGprofile latestPixelProfile = cgGLGetLatestProfile(CG_GL_FRAGMENT);

  if (latestGeometryProfile == CG_PROFILE_UNKNOWN) {
    cerr << "ERROR: geometry profile is not available" << endl;
    exit(0);
  }

  cgGLSetOptimalOptions(latestGeometryProfile);
  CGerror Error = cgGetError();
  if (Error) {
	  cerr << "CG ERROR: " << cgGetErrorString(Error) << endl;
  }

  cout << "Info: Latest VP Profile Supported: " << cgGetProfileString(latestVertexProfile) << endl;
  cout << "Info: Latest GP Profile Supported: " << cgGetProfileString(latestGeometryProfile) << endl;
  cout << "Info: Latest FP Profile Supported: " << cgGetProfileString(latestPixelProfile) << endl;

  vertexCGprofile = latestVertexProfile;
  geometryCGprofile = latestGeometryProfile;
  pixelCGprofile = latestPixelProfile;
  cgContext = cgCreateContext();  

  needInit = false;

}

bool ShaderOneInterface::PerSessionInit(CGInterface *cgi) {

  geometryProgram = cgCreateProgramFromFile(cgi->cgContext, CG_SOURCE, 
    "CG/shaderOne.cg", cgi->geometryCGprofile, "GeometryMain", NULL);
  if (geometryProgram == NULL)  {
    CGerror Error = cgGetError();
    cerr << "Shader One Geometry Program COMPILE ERROR: " << cgGetErrorString(Error) << endl;
    cerr << cgGetLastListing(cgi->cgContext) << endl << endl;
    return false;
  }

  vertexProgram = cgCreateProgramFromFile(cgi->cgContext, CG_SOURCE, 
    "CG/shaderOne.cg", cgi->vertexCGprofile, "VertexMain", NULL);
  if (vertexProgram == NULL) {
    CGerror Error = cgGetError();
    cerr << "Shader One Vertex Program COMPILE ERROR: " << cgGetErrorString(Error) << endl;
    cerr << cgGetLastListing(cgi->cgContext) << endl << endl;
    return false;
  }

  fragmentProgram = cgCreateProgramFromFile(cgi->cgContext, CG_SOURCE, 
    "CG/shaderOne.cg", cgi->pixelCGprofile, "FragmentMain", NULL);
  if (fragmentProgram == NULL)  {
    CGerror Error = cgGetError();
    cerr << "Shader One Fragment Program COMPILE ERROR: " << cgGetErrorString(Error) << endl;
    cerr << cgGetLastListing(cgi->cgContext) << endl << endl;
    return false;
  }

	// load programs
	cgGLLoadProgram(geometryProgram);
	cgGLLoadProgram(vertexProgram);
	cgGLLoadProgram(fragmentProgram);

	// build some parameters by name such that we can set them later...
  geometryScaleFactor = cgGetNamedParameter(geometryProgram, "scaleFactor" );
  geometryModelViewProj = cgGetNamedParameter(geometryProgram, "modelViewProj" );
  vertexModelViewProj = cgGetNamedParameter(vertexProgram, 
    "modelViewProj" );
  fragmentColor = cgGetNamedParameter(fragmentProgram, "highlightColor" );
  fragmentEye = cgGetNamedParameter(fragmentProgram, "eye" );
  fragmentbbA = cgGetNamedParameter(fragmentProgram, "bbA" );
  fragmentbbB = cgGetNamedParameter(fragmentProgram, "bbB" );
  fragmentbbC = cgGetNamedParameter(fragmentProgram, "bbC" );

  return true;

}

void ShaderOneInterface::PerFrameInit() {

	//set parameters
	cgGLSetStateMatrixParameter(vertexModelViewProj, 
		CG_GL_MODELVIEW_PROJECTION_MATRIX, 
    CG_GL_MATRIX_IDENTITY);

  float scaleFactor = 0.25f;
  cgGLSetParameter1f(geometryScaleFactor, scaleFactor);
	cgGLSetStateMatrixParameter(geometryModelViewProj, 
		CG_GL_MODELVIEW_PROJECTION_MATRIX, 
    CG_GL_MATRIX_IDENTITY);


  V3 highlightColor(1.0f, 0.0f, 0.0f);
  cgGLSetParameter3f(fragmentColor, 
    highlightColor[0], highlightColor[1], highlightColor[2]);
  cgGLSetParameter3f(fragmentEye, 
    scene->ppc->C[0], scene->ppc->C[1], scene->ppc->C[2]);
  cgGLSetParameter3f(fragmentbbA, 
    scene->tmeshes[1].verts[0][0], 
    scene->tmeshes[1].verts[0][1], 
    scene->tmeshes[1].verts[0][2]);

}

void ShaderOneInterface::BindPrograms() {

  cgGLBindProgram( geometryProgram);
  cgGLBindProgram( vertexProgram);
  cgGLBindProgram( fragmentProgram);

}

void CGInterface::DisableProfiles() {

  cgGLDisableProfile(vertexCGprofile);
  cgGLDisableProfile(geometryCGprofile);
  cgGLDisableProfile(pixelCGprofile);

}

void CGInterface::EnableProfiles() {

  cgGLEnableProfile(vertexCGprofile);
  cgGLEnableProfile(geometryCGprofile);
  cgGLEnableProfile(pixelCGprofile);

}




